# download mlflow model
import json
import logging
import tempfile
from os import environ
from pathlib import Path
from typing import List, Optional

from botocore.exceptions import ClientError, NoCredentialsError, PartialCredentialsError
from mlflow import MlflowException
from mlflow import artifacts as mlflow_artifacts
from python_on_whales import docker

logger = logging.getLogger(__name__)


def _download_model_artifact(artifact_uri: str, path: Path) -> None:
    """Get the model URI from the tracking server and download it.

    Args:
        artifact_uri (str): The uri for the artifact to download
        path (Path): Where to download the model to

    Raises:
        ValueError: Download failed because of a known configuration issue.
        MlflowException: Download failed because of an unhandled MLflow error.
    """
    try:
        mlflow_artifacts.download_artifacts(artifact_uri=artifact_uri, dst_path=path / "models")
    except MlflowException as e:
        if not environ.get("MLFLOW_TRACKING_URI"):
            raise ValueError(
                "Could not reach the MLflow tracking server because MLFLOW_TRACKING_URI was not set."
            ) from e
        raise e
    except (NoCredentialsError, PartialCredentialsError) as e:
        raise ValueError(
            "Tried to download artifact from AWS S3, but found no complete credentials set."
        ) from e
    except ClientError as e:
        raise ValueError(
            "The AWS SDK could not reach the resource because of client configuration errors."
        ) from e
    logger.debug("Downloaded model %s to %s", artifact_uri, path)


def _create_mode_settings_file(run_id: str, path: Path) -> None:
    model_config = {
        "name": "mlflow-model",
        "implementation": "mlserver_mlflow.MLflowRuntime",
        "parameters": {
            "uri": "/app/models/model",
            "version": run_id,
        },
    }

    with open(path / "model-settings.json", "w") as f:
        json.dump(model_config, f)

    logger.debug("Wrote model config to %s: %s", path, model_config)


def _build_model_service_image(path: Path, tags: Optional[List[str]] = None):
    if not tags:
        logger.warning("No tags provided, will build image without tags.")
    else:
        logger.debug("Found provided tags %s", tags)

    image = docker.build(
        path,
        tags=tags,
        file=Path(__file__).parent / "Dockerfile",
    )
    logger.debug("Successfully built image %s", image.id)


def download_and_build(artifact_uri: str, tag: Optional[List[str]]):
    """Download model artifact to local disc and build into docker image.

    Args:
        artifact_uri (str): URI of the artifact to download
        tag (Optional[List[str]]): List of tags to apply to the built image
    """
    with tempfile.TemporaryDirectory() as tmpdir:
        tmpdir_path = Path(tmpdir)
        _download_model_artifact(artifact_uri, tmpdir_path)
        _create_mode_settings_file(artifact_uri, tmpdir_path)
        _build_model_service_image(tmpdir_path, tag)



from dotenv import load_dotenv

# Keep this in sync with pyproject.toml manually
# Or use this https://github.com/tiangolo/poetry-version-plugin
# after this is resolved: https://github.com/tiangolo/poetry-version-plugin/issues/27
__version__ = "0.4.1"

load_dotenv()



import logging
from typing import List, Optional

import typer
from mlflow import __version__ as mlflow_version

from mlflow_mlserver_docker.main import download_and_build

from . import __version__

logging.basicConfig(level=logging.DEBUG)

app = typer.Typer(
    help="Simple CLI tool that helps you package an MLflow model on an MLflow tacking server into a docker image capable of serving an mlserver-compliant inference webservice."
)


@app.command(help="Display the version")
def version():
    """Display the version information."""
    print("mlflow-mlserver-docker: ", __version__)
    print("mlflow: ", mlflow_version)


@app.command(help="Download model artifact and build into mlserver image")
def build(  # noqa: D103
    artifact_uri: str = typer.Argument(  # noqa: B008
        ...,
        help="See https://www.mlflow.org/docs/latest/python_api/mlflow.artifacts.html for possible formats",
    ),
    tag: Optional[List[str]] = typer.Option(  # noqa: B008
        None,
        help="Tag the image (multiple values allowed!), otherwise no tag will be used",
    ),
    log_level: str = typer.Option("INFO"),  # noqa: B008
):
    logging.getLogger().setLevel(log_level)
    download_and_build(artifact_uri, tag)


if __name__ == "__main__":
    app()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['mlflow_mlserver_docker']

package_data = \
{'': ['*']}

install_requires = \
['boto3>=1.24.76,<2.0.0',
 'mlflow-skinny>=1.28.0,<2.0.0',
 'python-dotenv>=0.21.0,<0.22.0',
 'python-on-whales>=0.52.0,<0.53.0',
 'typer[all]>=0.6.1,<0.7.0']

entry_points = \
{'console_scripts': ['mlflow-mlserver-docker = mlflow_mlserver_docker.cli:app']}

setup_kwargs = {
    'name': 'mlflow-mlserver-docker',
    'version': '0.4.1',
    'description': 'Package mlflow models as mlserver docker images',
    'long_description': "# Mlflow Mlserver Docker\n\nThis is a utility package for building mlserver docker images based on MLflow models on a remote\ntracking server.\n\nCurrently, the S3 backend is the only officially supported backend, but any backend should work just\nfine as long as you have all the required dependencies installed. If you're interested in other\nbackends let me know.\n\n## Installation\n\n```\npip install mlflow-mlserver-docker\n```\n\n## Usage\n\n```sh\nmlflow-mlserver-docker build runs:/fg8934ug54eg9hrdegu904/model\n```\n\n## Development\n\n```sh\npoetry install\npoetry self add poetry-version-plugin\n```\n\nMake sure `docker` is running.\n",
    'author': 'Martin Morset',
    'author_email': 'mmorset@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/dingobar/mlflow-mlserver-docker',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
